@@ -14,7 +14,6 @@ if (ggplot2Avail){
   blanklabs <- labs(x = '', y = '')  
 }
 primenum <- c(83,89,97,101,103,107,109,113,127,131,137,139,149,151,157,163,167,173,179,181,191,193,197,199)
-# reduce.margins <- theme(axis.title = element_blank(),plot.margin=unit(c(0.25,0.25,0,0),"lines"));
 lsize=0.5;
 hsize=1;
 xcat <- function(obj){
@@ -45,37 +44,52 @@ shinyServer(function(input, output, session) {
    alpha <- sample(seq(from = input$alpha[1], to = input$alpha[2], length = m), m);
 	 shape <- sample(seq(from = input$gshape[1], to = input$gshape[2], length = m), m);
 	 scale <- sample(seq(from = input$gscale[1], to = input$gscale[2], length = m), m);
-	 BA    <- 1/sqrt(sample(primenum, m))
+	 width <- 1/sqrt(sample(primenum, m))
 	 G     <- switch(input$blur, 
               'smooth'= gammaBlur(n, shape, scale),
-              'direct' = directBlur(c(n, m)), 
-              'box.car'= boxcarBlur(n, BA))
+              'direct' = directBlur(n, m), 
+              'box.car'= boxcarBlur(n, width))
 	 
 	 signal   <- switch(input$sig,
               'lidar' = makeLIDAR(n), 
 				      'doppler' = makeDoppler(n),
               'bumps' = makeBumps(n),
-              'blocks' = makeBlocks(n))
+              'blocks' = makeBlocks(n),
+              'cusp' = makeCusp(n),
+              'heavisine' = makeHeaviSine(n))
 	 signalName <- switch(input$sig,'lidar' = "LIDAR Signal",
 				'doppler' = "Doppler Signal",
 				'bumps' = "Bumps Signal",
-				'blocks' = "Blocks Signal")
+				'blocks' = "Blocks Signal",
+        'cusp' = "Cusp Signal",
+        'heavisine' = "HeaviSine Signal")
 	 signalBlur <- blurSignal(signal, G)
    sigma <- sigmaSNR(signalBlur, SNR)
    eps <- multiNoise(n, sigma, alpha)
 	 Y   <- signalBlur + eps;
-	 x   <- (1:n)/n;
-	 
-   list(m = m, n = n, signal = Y, G = G, blur = input$blur, SNR = SNR, alpha = alpha, shape = shape, 
-        scale = scale , G = G, trueSignal = signal, eps = eps, signalBlur = signalBlur, BA = BA, 
-        x = x, signalName = signalName, sigma = sigma)
+	 x   <- seq(from = 0, to = 1 - 1/n, length = n);
+	 resolution <- input$resolution
+   
+   list(m = m, n = n, signal = Y, G = G, blur = input$blur, resolution = input$resolution, SNR = SNR, 
+        alpha = alpha, shape = shape, scale = scale , G = G, trueSignal = signal, eps = eps, 
+        signalBlur = signalBlur, width = width, x = x, signalName = signalName, sigma = sigma)
   })
   
   mWaveDList <- reactive({
     
     multiSig <- sigList()
-    mlwvd <- multiWaveD(multiSig$signal, multiSig$G, alpha = multiSig$alpha, 
-                        blur = multiSig$blur, shrinkType = input$shrinkage1, deg = as.integer(input$degree))
+    if (input$etaChoose == "TRUE"){
+      eta <- input$eta
+      mlwvd <- multiWaveD(multiSig$signal, multiSig$G, alpha = multiSig$alpha, eta = eta,
+                          resolution = multiSig$resolution, shrinkType = input$shrinkage1,
+                          deg = as.integer(input$degree))
+    } else {
+      mlwvd <- multiWaveD(multiSig$signal, multiSig$G, alpha = multiSig$alpha, 
+                          resolution = multiSig$resolution, shrinkType = input$shrinkage1,
+                          deg = as.integer(input$degree))
+    }
+    
+
     
     j0 <- mlwvd$j0
     j1 <- mlwvd$j1
@@ -139,44 +153,9 @@ shinyServer(function(input, output, session) {
   output$multiPlot <- renderPlot({
     mList <- mWaveDList()
     
-    j0 <- mList$j0
-    j1 <- mList$j1
-    
-    js <- rep(j0:j1, 2^(j0:j1))
-    ks <- unlist(lapply(j0:j1, function(i) (0:(2^i-1)/2^i)))
-    wi <- (2^j0 + 1):2^(j1 + 1)
-    nw <- length(wi)
-    w  <- mList$coef$coef[wi]
-    wf <- 2.1 * max(w)
-    w  <- w/wf
-    wc <- mList$shrinkCoef$coef[wi]/wf
-    
-    
-    raw.size = 1
-    shrink.size = 1
-    if (ggplot2Avail){
-      mra.df <- data.frame(w = c(w + js, wc+js), js = rep(js, 2), ks = rep(ks , 2), MRA = rep(LETTERS[1:2], each = nw), wsize = rep(c(raw.size, shrink.size), each = nw), method = rep(c('Raw', paste(mList$mWaveD$shrinkType, ' Thresholding', sep = '')), each = nw))
-      
-      mra.plot <- ggplot(mra.df) + geom_segment(aes(x=ks, y=js,xend=ks,yend=w,colour=MRA,size=wsize)) + labs(x="k",y='j') +scale_color_discrete(labels= c('Raw',paste(mList$mWaveD$shrinkType,' Thresholding',sep='')),guide=guide_legend(title.position='left',title.theme = element_text(size=15,angle=0))) + scale_size(guide='none') + guides(colour = guide_legend(override.aes = list(size = 10), title='Multiresolution Analysis')) + theme(legend.position="top",legend.key=element_rect(fill=NA), axis.text.y = element_text(angle=90))  + scale_y_continuous(breaks = j0:j1)
-      print(mra.plot)
-    } else {
-      buf <- 0.5
-      thickness <- 4
-      plot(0, type = "n", xlim = c(0,1), ylim = c(j0 - buf, j1 + buf), yaxt = 'n', xlab = "",
-           ylab = "Resolution Level", main = "MultiResolution Analysis of Coef.")
-      axis(2, at = j0:j1)
-      ws <- w + js
-      survived <- which(wc != 0)
-      kss <- ks[survived]
-      jss <- js[survived]
-      wss <- wc[survived] + jss
-      segments(ks, js, ks, ws, lwd = thickness, col = 'red')
-      segments(kss, jss, kss, wss, lwd = thickness, col = 'blue')
-      par(fig = c(0, 1, 0, 1), oma = c(0, 0, 0, 0), mar = c(0, 0, 0, 0), new = TRUE)
-      legend("bottom", c('Raw',paste(mList$mWaveD$shrinkType,' Thresholding',sep='')), xpd = TRUE, horiz = TRUE, 
-             inset = -0.1, bty = "n", lty = rep(1,2), col = c('red','blue'), cex = 1)
-    }
+    mWaveD <- mList$mWaveD
     
+    plot(mWaveD, which = 4)
   },  height=function() { session$clientData$output_multiPlot_width * 3/4 })
 
 output$wvdPlot <- renderPlot({
@@ -200,29 +179,29 @@ output$wvdPlot <- renderPlot({
 	    k <- 3
 	    extra <- rep('dashed', n)
 	    if (m == 1){
-	      out = c(out,multiEstimate(mlwvd$signal , G = mlwvd$G , blur = 'smooth', alpha = mlwvd$alpha, shrinkType = mlwvd$shrinkType, deg = as.integer(input$degree)))
+	      out = c(out,multiEstimate(mlwvd$signal , G = mlwvd$G , resolution = mlwvd$resolutionMethod, alpha = mlwvd$alpha, shrinkType = mlwvd$shrinkType, deg = as.integer(input$degree)))
 	      if (input$wvdshow == 2){
 	        plot.lab <- 'Naive mWaveD average'
 	      } else {
 	        plot.lab <- 'Best channel'
 	      }
-	      group <- c(group,rep(plot.lab,n))
+	      group <- c(group, rep(plot.lab, n))
 	      lty <- c(lty,extra)
 	    } else {
 	      if (input$wvdshow == 2){
 	        wvdMean = apply(sapply(1:m, function(x) 
-	          multiEstimate(as.matrix(mlwvd$signal[, x]) ,as.matrix(mlwvd$G[, x]) , blur = 'smooth', alpha = mlwvd$alpha[x], shrinkType = mlwvd$shrinkType, deg = as.integer(input$degree))), 1, mean)
+	          multiEstimate(as.matrix(mlwvd$signal[, x]) ,as.matrix(mlwvd$G[, x]) , resolution = mlwvd$resolutionMethod, alpha = mlwvd$alpha[x], shrinkType = mlwvd$shrinkType, deg = as.integer(input$degree))), 1, mean)
 	        out <- c(out,wvdMean)
 	        group <- c(group,rep('Naive mWaveD average',n))
 	        lty <- c(lty,extra)
 	      } else {
-	        blur.ty <- mlwvd$blurType
-	        if (blur.ty == 'smooth'){
-	          best <- mlwvd$blurInfo$bestChannel
-	        } else {
-	          best <- which.min(mlwvd$sigmaEst)
+	        resolution.ty <- mlwvd$resolution
+	        if (resolution.ty == 'smooth'){
+	          best <- which.max(mlwvd$blurInfo$freqCutoffs)
+          } else {
+	          best <- which.min(mlwvd$sigmaEst)  
 	        }
-          wvdBest <- multiEstimate(as.matrix(mlwvd$signal[, best]), as.matrix(mlwvd$G[, best]), blur = 'smooth', deg = as.integer(input$degree))
+          wvdBest <- multiEstimate(as.matrix(mlwvd$signal[, best]), as.matrix(mlwvd$G[, best]), resolution = mlwvd$resolutionMethod, deg = as.integer(input$degree))
 	        out <- c(out, wvdBest)
 	        group <- c(group, rep("mWaveD best chan", n))
 	        lty <- c(lty, extra)
@@ -240,8 +219,8 @@ output$wvdPlot <- renderPlot({
           multiEstimate(as.matrix(mlwvd$signal[, x]) ,as.matrix(mlwvd$G[, x]) , blur = 'smooth', alpha = mlwvd$alpha[x], shrinkType = mlwvd$shrinkType, deg = as.integer(input$degree))), 1, mean)
         xtralab <- 'Naive mean'
       } else {
-        blur.ty <- mlwvd$blurType
-        if (blur.ty == 'smooth'){
+        resolution.ty <- mlwvd$resolution
+        if (resolution == 'smooth'){
           best = mlwvd$blurInfo$bestChannel
         } else {
           best = which.min(mlwvd$sigmaEst)
@@ -274,101 +253,11 @@ output$wvdPlot <- renderPlot({
     iw = -(n/2 - 1):(n/2)
     
     blurInfo = mWaveD$blurInfo
-    blurType = mWaveD$blurType
+    resType = mWaveD$resolutionMethod
+    blurDetected = mWaveD$blurDetected
+    
+    plot(mWaveD, which = 3)
     
-    if (blurType != "box.car"){
-      blurf <- blurInfo$decay
-      cutf <- blurInfo$cutoff
-      ymin <- min(cutf)
-      revblur <- as.matrix(blurf[(n/2):2, ])
-      revcut <- as.matrix(cutf[(n/2):2, ])
-      tS <- (1:(n/2))
-      
-      tS <- c(-rev(tS[-(n/2)]), 0, tS)
-      blur <- rbind(revblur, blurf)
-      cut <- rbind(revcut, cutf)
-      plotTitle <- 'Decay of weighted Kernel coefficients in Fourier domain'
-      xlab <- 'Fourier freq'
-      if (ggplot2Avail){
-        fourier.df <- data.frame(Y = as.vector(blur), x = rep(iw,m), Ycut = as.vector(cut),Channel=rep(LETTERS[1:m],each=n),m=m)
-        resolution.plot <- ggplot(fourier.df) + geom_line(aes(x=x, y=Y, colour=Channel,group=Channel),size=hsize) + geom_line(aes(x=x, y=Ycut, colour=Channel), linetype='dashed', size=hsize) + ggtitle(plotTitle) + labs(x = xlab, y = '')
-        if (blurType != 'smooth'){
-          print(resolution.plot)
-        }
-      }
-      if (blurType == 'smooth'){
-        xbest <- max(blurInfo$freqCutoffs) - 1
-        ybest <- blurf[xbest, blurInfo$bestChannel]
-        xlim <- min(2 * xbest, n/2)
-        if (ggplot2Avail){
-          high.df <- data.frame(x = rep(xbest, 2), y = c(-Inf, ybest))
-          point.df <- data.frame(xbest = xbest, ybest = ybest)
-          resolution.plot <- resolution.plot + geom_line(aes(x = x, y = y), linetype = 'dotted' , data = high.df) + geom_line(aes(x = -x + 1, y = y), linetype = 'dotted' , data = high.df) + geom_point( aes(x = xbest, y = ybest), size = 4, shape = 1, data = point.df) + geom_point( aes(x = -xbest + 1, y = ybest), size = 4, shape = 1, data = point.df)
-        } 
-        ylimlow <- 0
-      } else {
-        xlim <- n/2
-        ylimlow <- 0.5
-      }
-      xlim <- c(-xlim, xlim)
-      l <- n/2 + 1
-      ylim <- c(min(cutf[2, ], blurf[l, ]), ylimlow)
-      
-      zoom = input$zoom
-      if (zoom != TRUE){
-        xlim = c(-n/2, n/2)
-      }
-      
-      if (ggplot2Avail){
-        resolution.plot <- resolution.plot + coord_cartesian(xlim = xlim, ylim = ylim)
-        print(resolution.plot)
-      } else {
-        par(oma = c(4, 1, 1, 1))
-        ylim <-  c(min(cutf[2, ], blurf[n/2 + 1, ]), 0.5)
-        if (blurType != 'smooth'){
-          xlim = c(-n/2, n/2)
-        }
-        matplot(iw, blur, type = 'l', main = plotTitle, xlab = 'Fourier freq', ylab = '', xlim = xlim, ylim = ylim, lty = 'solid')
-        matlines(iw, cut, lty = 'dashed')
-        if (blurType == 'smooth'){
-          points(xbest, ybest, col='blue')
-          points(-xbest + 1, ybest, col = 'blue')
-          xbest <- rep(xbest, 2)
-          ybest <- c(ylim[1], ybest)
-          lines(xbest, ybest, lty = 'dotted')
-          lines(-xbest + 1, ybest, lty = 'dotted')
-        }
-        grid()
-        par(fig = c(0, 1, 0, 1), oma = c(0, 0, 0, 0), mar = c(0, 0, 0, 0), new = TRUE)
-        legend("bottom", paste('Channel ', LETTERS[1:m]), xpd = TRUE, horiz = TRUE, 
-               inset = -0.1, bty = "n", lty = 1:4, col = 1:4, cex = 1)
-      }
-    } else {
-      j0   <- mWaveD$j0
-      j1   <- mWaveD$j1
-      J    <- floor(log2(n)) - 1
-      j    <- j0:min(c(J - 1, 2 * j1))
-      blkV <- blurInfo$blockVar[1:length(j)]
-      blkc <- blurInfo$blockCutoff[1:length(j)]
-      ylim <- range(c(blurInfo$blockVar, blurInfo$blockCutoff))
-      if (ggplot2Avail){
-        resolution.df <- data.frame(Y = c(blkV, blkc), x = rep(j,2), colour = rep(c("Resolution var.",'Resolution bounds'), each = length(j)) , Ycut = blkc)
-        bestV <- blkV[j == j1]
-        high.df <- data.frame(x = c(j1, j1), y = c(ylim[1], bestV))
-        point.df <- data.frame(j1 = j1, bestV = bestV)
-        resolution.plot <- ggplot(resolution.df) + geom_line(aes(x = x, y = Y, colour = colour, linetype = colour), size = hsize) +  geom_line(aes(x = x, y = y), linetype = 'dotted', data = high.df) + labs(x = 'j', y = '') + geom_point( aes(x = j1, y = bestV), size = 4, shape = 1, data = point.df)  + scale_color_discrete(labels= c('Resolution bounds', 'Resolution var.'), guide=guide_legend(title.position='left',title.theme = element_text(size=15,angle=0))) + scale_size(guide='none') + guides(colour = guide_legend( title='Blockwise resolution decay')) + theme(legend.position="top", legend.key = element_rect(fill = NA), axis.text.y = element_text(angle = 90)) + scale_linetype_manual(values=c(1,2), name="Blockwise resolution decay", labels=c('Resolution bounds', 'Resolution var.')) + scale_x_continuous(breaks = j)
-        print(resolution.plot)
-      } else {
-        plot(j, blkV, type = 'b', xlab = 'j', ylab = '', main = 'Block wise resolution selection')
-        lines(j, blkc, col = 2)
-        points(j1, blurInfo$blockVar[j == j1], col='blue')
-        lines(c(j1, j1), c(ylim[1], blurInfo$blockVar[j == j1]), lty = 'dashed')
-        grid()
-        legend("top", c('Resolution bounds', 'Resolution var.'), xpd = TRUE, horiz = TRUE, 
-               inset = 0, bty = "n", pch = c(NA_integer_,21) , lty = rep(1,2), col = 2:1, cex = 1.2)
-      }
-      
-    }
   },  height=function() { session$clientData$output_resolutionPlot_width * 3/4 })
 
 	output$summaryOut <- renderPrint({ 
@@ -415,29 +304,25 @@ output$wvdPlot <- renderPlot({
     cat("\n\n")
     cat("Estimated Channel information:\n\n")
     
-    if (wList$mWaveD$blurType == "direct"){
-      mat <- cbind(round(wList$mWaveD$sigmaEst, 3), round(wList$mWaveD$alpha, 3), wList$mWaveD$blurInfo$freq, rep(wList$mWaveD$blurInfo$j1, wList$m))
-      colnames(mat) <- c("sigma", "alpha", "cutoff", "best res")
-      rownames(mat) <- paste("Chan ", 1:wList$m,':', sep = '')
-      print(mat)
-    } else {
-      # If Smooth blur is used, display the matrix of values
-      if (wList$mWaveD$blurType == "smooth"){
+    if (wList$mWaveD$resolutionMethod == "smooth"){
+      if (wList$mWaveD$blurDetected == "direct") {
+        mat <- cbind(round(wList$mWaveD$sigmaEst, 3), round(wList$mWaveD$alpha, 3), wList$mWaveD$blurInfo$freq, rep(wList$mWaveD$j1, wList$m))
+        colnames(mat) <- c("sigma", "alpha", "cutoff", "best res")
+        rownames(mat) <- paste("Chan ", 1:wList$m,':', sep = '')  
+        print(mat)
+      } else {
         mat <- cbind(round(wList$mWaveD$sigmaEst, 3), round(wList$mWaveD$alpha, 3), wList$mWaveD$blurInfo$freq, wList$mWaveD$blurInfo$maxLevels)
         colnames(mat) <- c("sigma", "alpha", "cutoff", "best res")
         rownames(mat) <- paste("chan: ", 1:wList$m)
         print(mat)
         cat("\n")
         cat("Estimated best channel = ", wList$mWaveD$blurInfo$bestChannel)
-        
-      } else {
-        if (wList$mWaveD$blurType == "box.car"){
-          mat <- cbind(round(wList$mWaveD$sigma, 3), round(wList$mWaveD$alpha, 3))
-          colnames(mat) <- c("sigma", "alpha")
-          rownames(mat) <- paste("chan: ", 1:wList$m)
-          print(mat)
-        }
       }
+    } else {
+      mat <- cbind(round(wList$mWaveD$sigma, 3), round(wList$mWaveD$alpha, 3))
+      colnames(mat) <- c("sigma", "alpha")
+      rownames(mat) <- paste("chan: ", 1:wList$m)
+      print(mat)
     }
   })
 
@@ -477,7 +362,7 @@ output$wvdPlot <- renderPlot({
       colnames(mat) <- c("sigma", "alpha","SNR", "shape", "scale")
     } else {
       if (sList$blur == 'box.car'){
-        mat <- cbind(mat, paste(round(sList$BA, 3), " = 1/sqrt(", 1/sList$BA^2 ,")", sep = '') )
+        mat <- cbind(mat, paste(round(sList$width, 3), " = 1/sqrt(", 1/sList$width^2 ,")", sep = '') )
         colnames(mat) <- c("sigma", "alpha","SNR", "Box.car widths")        
       }
     }
@@ -495,14 +380,16 @@ output$wvdPlot <- renderPlot({
            'lidar' = cat('signal <- makeLIDAR(n)', '\n'),
            'doppler' = cat('signal <- makeDoppler(n)', '\n'),
            'bumps' = cat('signal <- makeBumps(n)', '\n'),
-           'blocks' = cat('signal <- makeBlocks(n)', '\n'))
+           'blocks' = cat('signal <- makeBlocks(n)', '\n'),
+           'cusp' = cat('signal <- makeCusp(n)\n'),
+           'heavisine' = cat('signal <- makeHeaviSine(n)\n'))
     cat('t <- (0:(n-1))/n\n')
     if (input$signalShow == '1'){
       cat("plot(t, signal, type = 'l')\n")
     }
     if (input$signalShow == '2' || input$signalShow == '3'){
       if (sList$blur == 'direct'){
-        cat('G <- directBlur(c(n, m))\n') 
+        cat('G <- directBlur(n, m)\n') 
       }
       if (sList$blur == 'smooth'){
         if (sList$m == 1){
@@ -516,9 +403,9 @@ output$wvdPlot <- renderPlot({
       }
       if (sList$blur == 'box.car'){
         if (sList$m == 1){
-          cat('boxWindow <- 1/sqrt(', 1/as.numeric(sList$BA)^2,')\n', sep = '')
+          cat('boxWindow <- 1/sqrt(', 1/as.numeric(sList$width)^2,')\n', sep = '')
         } else {
-          cat('boxWindow <- 1/sqrt(', xcat(1/as.numeric(sList$BA)^2),')\n', sep = '')
+          cat('boxWindow <- 1/sqrt(', xcat(1/as.numeric(sList$width)^2),')\n', sep = '')
         }
         cat("G <- boxcarBlur(n, boxWindow)\n")
       }
@@ -528,14 +415,18 @@ output$wvdPlot <- renderPlot({
       }
       if (input$signalShow == '3'){
         if (sList$m == 1){
-          cat('alpha <-', numclean(sList$alpha),'\n')
+          if (any(input$alpha != 1)){
+            cat('alpha <-', numclean(sList$alpha),'\n')
+          }
           cat('SNR <-', numclean(sList$SNR), '\n')
         } else {
-          cat('alpha <-', xcat(numclean(sList$alpha)),'\n')
+          if (any(input$alpha != 1)){
+            cat('alpha <-', xcat(numclean(sList$alpha)),'\n')  
+          }
           cat('SNR <-', xcat(numclean(sList$SNR)), '\n')  
         }
         cat('sigma <- sigmaSNR(X, SNR)\n')
-        cat('E <- multiNoise(n, sigma, alpha)\n')
+        cat('E <- multiNoise(n, sigma, ',ifelse(any(input$alpha != 1), paste0('alpha'),),')\n', sep ='')
         cat('Y <- X + E\n')
         cat("matplot(t, Y, type = 'l')")
       }
@@ -546,7 +437,12 @@ output$wvdPlot <- renderPlot({
     cat('Base R Function calls:\n\n')
     
     wList <- mWaveDList()
-    cat("mWaveD.output <- multiWaveD(Y, G = G, alpha = alpha, blur = '",wList$mWaveD$blurType,"', shrinkType = '",wList$mWaveD$shrinkType,"', deg = ",wList$mWaveD$degree,')\n', sep = '')
+    showG <- ifelse(input$blur != "direct", paste0(",G = G"), paste0(""))
+    showAlpha <- ifelse(any(input$alpha != 1), paste0(", alpha = alpha"), paste0(""))
+    showRes <- ifelse((wList$mWaveD$resolution == "block" && detectBlur(wList$mWaveD$G) != "box.car") || (wList$mWaveD$resolution == "smooth" && detectBlur(wList$mWaveD$G) == "box.car"), paste0(", resolution = '",wList$mWaveD$resolutionMethod,"'"),paste0(""))
+    showShrink <- ifelse(wList$mWaveD$shrinkType != "hard", paste0(", shrinkType = '",wList$mWaveD$shrinkType,"'"),paste0(""))
+    showDeg <- ifelse(wList$mWaveD$degree != 3,  paste0(", deg = ",wList$mWaveD$degree), paste0(""))
+    cat("mWaveD.output <- multiWaveD(Y", showG, showAlpha, showRes, showShrink, showDeg,')\n', sep = '')
     cat('plot(mWaveD.output, which = 3)')
   })
 
@@ -554,12 +450,17 @@ output$wvdPlot <- renderPlot({
     cat('Base R Function calls:\n\n')
 
     wList <- mWaveDList()
+    showG <- ifelse(input$blur != "direct", paste0(",G = G"), paste0(""))
+    showAlpha <- ifelse(any(input$alpha != 1), paste0(", alpha = alpha"), paste0(""))
+    showRes <- ifelse((wList$mWaveD$resolution == "block" && detectBlur(wList$mWaveD$G) != "box.car") || (wList$mWaveD$resolution == "smooth" && detectBlur(wList$mWaveD$G) == "box.car"), paste0(", resolution = '",wList$mWaveD$resolutionMethod,"'"),paste0(""))
+    showShrink <- ifelse(wList$mWaveD$shrinkType != "hard", paste0(", shrinkType = '",wList$mWaveD$shrinkType,"'"),paste0(""))
+    showDeg <- ifelse(wList$mWaveD$degree != 3,  paste0(", deg = ",wList$mWaveD$degree), paste0(""))
     cat('# Note the following commands just give the mWaveD estimate only.\n')
-    cat("mWaveD.output <- multiWaveD(Y, G = G, alpha = alpha, blur = '",wList$mWaveD$blurType,"', shrinkType = '",wList$mWaveD$shrinkType,"', deg = ",wList$mWaveD$degree,')\n', sep = '')
+    cat("mWaveD.output <- multiWaveD(Y", showG, showAlpha, showRes, showShrink, showDeg,')\n', sep = '')
     cat('plot(mWaveD.output, which = 2)\n')
     cat('## Or alternatively\n')
     cat('x = (0:(n-1))/n\n')
-    cat("estimate <- multiEstimate(Y, G = G, alpha = alpha, blur = '",wList$mWaveD$blurType,"', shrinkType = '",wList$mWaveD$shrinkType,"', deg = ",wList$mWaveD$degree,')\n', sep = '')
+    cat("estimate <- multiEstimate(Y", showG, showAlpha, showRes, showShrink, showDeg,')\n', sep = '')
     cat("plot(x, estimate, type = 'l')")
   })
 
@@ -598,6 +499,13 @@ output$wvdPlot <- renderPlot({
                   selected = currentShrinkage)
   updateSelectInput(session, inputId = 'shrinkage2', label = shrinkLabel, choices = shrinkChoices,
                   selected = currentShrinkage)
+
+  output$etaSlider <- renderUI({
+    sList <- sigList()
+    deta <- theoreticalEta(alpha = sList$alpha, blur = sList$blur, G = sList$G, sList$sigma)
+    sliderInput("eta", "Smoothing parameter range:",
+                min = 0.1, max = 4, value = deta)
+  })
 })
 
 
